package gov.va.vss.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import gov.va.vss.model.donation.DonationLog;
import gov.va.vss.service.DonationLogService;

@Service
public class DonationLogServiceImpl extends AbstractServiceImpl implements DonationLogService {
	private static final Logger log = LoggerFactory.getLogger(DonationLogServiceImpl.class);

	private SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();

	@Override
	public List<DonationLog> importExternalDonations(InputStream is)
			throws ParserConfigurationException, SAXException, IOException {
		SAXParser saxParser = saxParserFactory.newSAXParser();
		MyHandler handler = new MyHandler();
		saxParser.parse(is, handler);
		List<DonationLog> empList = handler.getResults();
		return empList;
	}

	static class MyHandler extends DefaultHandler {

		static enum Tag {
			TRACKING_ID, TRANSACTION_STATUS, TRANSACTION_DATE, TRANSACTION_AMOUNT, FACILITY, NAME, ADDRESS, CITY, STATE, ZIP, PHONE, EMAIL, PROGRAM_FIELD, ADDITIONAL_INFO, DEPOSIT_NUMBER;
		}

		private String type; // credit_card, ach_debit, etc
		private List<DonationLog> donationList = new ArrayList<>();
		private DonationLog current = null;

		// getter method for employee list
		public List<DonationLog> getResults() {
			return donationList;
		}

		private Tag currentTag;

		@Override
		public void startElement(String uri, String localName, String qName, Attributes attributes)
				throws SAXException {

			if (qName.equalsIgnoreCase("credit_card") || qName.equalsIgnoreCase("ach_debit")) {
				type = qName;
			}

			if (qName.equalsIgnoreCase("item_detail_record")) {
				// String id = attributes.getValue("id");
				current = new DonationLog();
				current.setType(type);
			} else if (qName.equalsIgnoreCase("paygov_tracking_id")) {
				currentTag = Tag.TRACKING_ID;
			} else if (qName.equalsIgnoreCase("transaction_status")) {
				currentTag = Tag.TRANSACTION_STATUS;
			} else if (qName.equalsIgnoreCase("transaction_date")) {
				currentTag = Tag.TRANSACTION_DATE;
			} else if (qName.equalsIgnoreCase("transaction_amount")) {
				currentTag = Tag.TRANSACTION_AMOUNT;
			} else if (qName.equalsIgnoreCase("Facility")) {
				currentTag = Tag.FACILITY;
			} else if (qName.equalsIgnoreCase("Name_Field")) {
				currentTag = Tag.NAME;
			} else if (qName.equalsIgnoreCase("Street_Address_Field")) {
				currentTag = Tag.ADDRESS;
			} else if (qName.equalsIgnoreCase("City_Field")) {
				currentTag = Tag.CITY;
			} else if (qName.equalsIgnoreCase("State1_Field")) {
				currentTag = Tag.STATE;
			} else if (qName.equalsIgnoreCase("ZipCode_Field")) {
				currentTag = Tag.ZIP;
			} else if (qName.equalsIgnoreCase("PhoneNumber_Field")) {
				currentTag = Tag.PHONE;
			} else if (qName.equalsIgnoreCase("account_holder_email_address")) {
				currentTag = Tag.EMAIL;
			} else if (qName.toLowerCase().startsWith("program_field_")) {
				currentTag = Tag.PROGRAM_FIELD;
			} else if (qName.equalsIgnoreCase("AdditionalInformation")) {
				currentTag = Tag.ADDITIONAL_INFO;
			} else if (qName.equalsIgnoreCase("deposit_ticket_number")) {
				currentTag = Tag.DEPOSIT_NUMBER;
			}
		}

		@Override
		public void endElement(String uri, String localName, String qName) throws SAXException {
			if (qName.equalsIgnoreCase("item_detail_record")) {
				donationList.add(current);
				current = null;
			}
		}

		@Override
		public void characters(char ch[], int start, int length) throws SAXException {
			if (currentTag == null || current == null)
				return;

			String strVal = length == 0 ? null : new String(ch, start, length).replaceAll("\\s+", " ").trim();

			switch (currentTag) {
			case TRACKING_ID:
				current.setTrackingId(strVal);
				break;
			case ADDITIONAL_INFO:
				current.setAdditionalInfo(strVal);
				break;
			case ADDRESS:
				current.setAddress(strVal);
				break;
			case CITY:
				current.setCity(strVal);
				break;
			case DEPOSIT_NUMBER:
				current.setDepositNumber(strVal);
				break;
			case EMAIL:
				current.setEmail(strVal);
				break;
			case FACILITY:
				current.setFacility(strVal);
				break;
			case NAME:
				current.setName(strVal);
				break;
			case PHONE:
				current.setPhone(strVal);
				break;
			case PROGRAM_FIELD:
				if (!"0".equals(strVal))
					current.setProgramField(strVal);
				break;
			case STATE:
				current.setState(strVal);
				break;
			case TRANSACTION_AMOUNT:
				current.setDonationAmount(length == 0 ? null : new BigDecimal(strVal));
				break;
			case TRANSACTION_DATE:
				current.setTransactionDate(length == 0 ? null : LocalDateTime.parse(strVal));
				break;
			case TRANSACTION_STATUS:
				current.setStatus(strVal);
				break;
			case ZIP:
				current.setZip(strVal);
				break;
			default:
				throw new AssertionError("Unexpected value: " + currentTag);
			}

			currentTag = null;
		}
	}
}
